"""
Constants for swarm-bus
"""

PREFIX = 'dev-%(hostname)s-'
PRIORITIES = ['low', 'medium', 'high']
OFFICE_HOURS = True
POLLING_INTERVAL = 20
RETENTION_PERIOD = 864000  # 10 days
VISIBILITY_TIMEOUT = 30
SLEEP_TIME = 0
REGION = 'eu-west-1'
